class DBConst:
    lendings = "lendings"
    borrows = "borrows"
    vaults = "vault"
    swaps = "swaps"
    providing_lps = "providinglps"
    id = "_id"
    launch_id = "launchId"
    chain_id = "chainId"
    name = "name"
    address = "address"
    config = "config"
    farming_type = "farmingType"
    token_in = "tokenIn"
    token_secondary = "tokenSecondary"
    token_out = "tokenOut"
    token_lp = "tokenLP"
    number_steps = "numberSteps"
    exchange_rate = "exchangeRate"
    reward_token = "rewardToken"
    roi_change_logs = "roiChangeLogs"
    current_roi = "currentROI"
    detail_roi_change_logs = "detailROIChangeLogs"
    current_detail_roi = "currentDetailROI"
    metadata = "metadata"
    url_link = "url"
    gas_token = "gasToken"
    gas_fee_change_logs = "gasFeeChangeLogs"
    current_gas_fee = "currentGasFee"
    last_updated_at = "lastUpdatedAt"
    interval_update = "intervalUpdate"
    convert_rate = "convertRate"
    liquidation_threshold = "liquidationThreshold"
    reward_deposit_apy = "rewardDepositAPY"
    reward_borrow_apy = "rewardBorrowAPY"
    type = "type"
    vaultType = "vaultType"
    created_at = "createdAt"
    deposit_apy = "depositAPY"
    borrow_apy = "borrowAPY"
    t_token = "tToken"
    d_token = "dToken"
    ltv = "ltv"
    reserves_list = "reservesList"
    is_new = "isNew"
    dapp_name = "dappName"
    project_name = "projectName"
    safety_index = "safetyIndex"
    amount = "amount"
    value_in_usd = "valueInUSD"
    closed_at = "closedAt"
    ended_at = "endedAt"
    swap_info_change_logs = "swapInfoChangeLogs"
    current_swap_info = "currentSwapInfo"
    pair_tokens = "pairTokens"
    coin_ids = "coinIds"
    token_in_total_supply = "tokenInTotalSupply"
    token_out_total_supply = "tokenOutTotalSupply"
    token_lp_total_supply = "tokenLPTotalSupply"
    lp_token = "lpToken"
    swap_fee = "swapFee"
    pid = "pid"
    holders = "currentNumberUsers"
    holders_change_logs = "numberUsersChangeLogs"
    safety_index_collections = [lendings, borrows, providing_lps, vaults, swaps]
    liquidity_change_logs = "liquidityChangeLogs"
    total_borrow = "totalBorrow"
    total_deposit = "totalDeposit"
    current_liquidity = "currentLiquidity"
    label = "label"
    pool_tokens = "poolTokens"
    pool_token_in = "poolTokenIn"
    pool_token_out = "poolTokenOut"
    v_token = "vToken"
    c_token = "cToken"
    disabled = "disabled"
    mint_paused = "mint_paused"
    borrow_paused = "borrow_paused"
