class ContractAddresses:
    WBNB = "0xbb4cdb9cbd36b01bd1cbaebf2de08d9173bc095c"
    BNB = "0x0000000000000000000000000000000000000000"
    TRAVA_BNB_LP = "0x865c77d4ff6383e06c58350a2cfb95cca2c0f056"
    R_TRAVA = "0x170772a06affc0d375ce90ef59c8ec04c7ebf5d2"
    VE_TRAVA_VAULT = "0xedf6a93772eecfb1300f61f6c1ae9680c33996a2"
    GENERAL_TRAVA_VAULT = "63daeb1c97e098480b16c7410dc4b9c0d4c401d6ff4735324edfbeaa"
    PANCAKE_FARM = "0x73feaa1ee314f8c655e354234017be2193c9e24e"
    # PANCAKE_FARM = "0xa5f8c5dbd5f286960b9d90548680ae5ebff07652"
    TRAVA_BSC_LENDING_POOL = "0x75de5f7c91a89c16714017c7443eca20c7a8c295"
    VALAS_BSC_LENDING_POOL = "0xe29a55a6aeff5c8b1beede5bcf2f0cb3af8f91f5"
    ALPACA_BSC_LENDING_POOL = "0xa625ab01b08ce023b2a342dbb12a16f2c8489a8f"
    DOGE = "0xba2ae424d960c26247dd6c32edc70b295c744c43"
    MAIN_STAKING_CONTRACT_PANCAKE_FARM = "0x73feaa1ee314f8c655e354234017be2193c9e24e"
    # MAIN_STAKING_CONTRACT_PANCAKE_FARM = "0xa5f8c5dbd5f286960b9d90548680ae5ebff07652"
    LUNA = "0xb91A659E88B51474767CD97EF3196A3e7cEDD2c8"
    UST = "0x78366446547D062f45b4C0f320cDaa6d710D87bb"
    BISWAP_FARM = "0xdbc1a13490deef9c3c12b44fe77b503c1b061739"


class WrappedNativeTokens:
    mapping = {
        "0x38": "0xbb4cdb9cbd36b01bd1cbaebf2de08d9173bc095c"
    }


class AbnormalVenusPool:
    venusDOGE = "0xec3422ef92b2fb59e84c8b02ba73f1fe84ed8d71"
    venusTRON = "0xc5d3466aa484b040ee977073fcf337f2c00071c1"
    decimals = {
        venusDOGE: 20,
        venusTRON: 24
    }


class AbnormalCreamPool:
    creamRenBTC = "0x7f746a80506a4cafa39938f7c08ad59cfa6de418"
    creamRenZEC = "0xbe7e1d74acae787355169bc61a8729b2040fce6b"
    decimals = {
        creamRenBTC: 20,
        creamRenZEC: 20
    }
